/*
	This is the main bit you'll need to set up
	 for anything you want to hide/unhide on Crossed/Uncrossed.
	Be aware that these aren't called when moving without side-effects
	 (for example, by setting loc directly).
*/
obj/tree
	// Hide trees upon a player crossing.
	Crossed(atom/movable/mover)
		astype(mover, /mob)?.client?.Hide(src)

	// Un-hide trees upon a player uncrossing.
	Uncrossed(atom/movable/mover)
		astype(mover, /mob)?.client?.Unhide(src)

// Below are all unrelated demo functionality and aesthetics.

proc
	astype(object, type)
		return istype(object, type) ? object : null

	lerp(a, b, t)
		return a * (1 - t) + b * t

world
	maxx = 50
	maxy = 50
	fps = 20
	view = "50x50"
	turf = /turf/grass
	mob = /mob/player
	map_format = SIDE_MAP

turf/grass
	icon = 'Shapes.dmi'
	icon_state = "rect"
	color = rgb(64, 128, 0)

	New()
		// Spawn trees on every grass turf with a certain chance.
		if(prob(2)) new/obj/tree(src)

obj/tree
	icon = 'Tree.dmi'
	layer = MOB_LAYER

	// These bounds determine where Crossing occurs within the tree's icon.
	bound_width = 256
	bound_height = 233

	New()
		// Randomize the tree's size.
		var
			old_width = bound_width
			old_height = bound_height
			width_multiplier = rand() * (1/2) + 3/4
			height_multiplier = rand() * (1/2) + 3/4
		transform *= matrix(width_multiplier, height_multiplier, MATRIX_SCALE)
		bound_width *= width_multiplier
		bound_height *= height_multiplier

		// Re-center the bounds.
		bound_x = (old_width - bound_width) / 2
		bound_y = (old_height - bound_height) / 2

mob/player
	icon = 'Shapes.dmi'
	icon_state = "oval"
	color = rgb(32, 64, 128)
	step_size = 16

	Login()
		// Move to the center of the map.
		Move(locate(round(world.maxx / 2, 1), round(world.maxy / 2, 1), 1))

		src << "Click and drag to move around."
		src << "Left mouse button for movement with side-effects."
		src << "Right mouse button for movement without side-effects."

	Stat()
		// This is to see that these are properly cleared when nothing's happening.
		stat("Images", client.images.len)
		stat("Hider", client.hider ? "\ref[client.hider]" : "null")
		stat("Fade Images", json_encode(client.hider?.fade_images))
		stat("Fade IDs", json_encode(client.hider?.fade_ids))

client
	fps = 60
	show_popup_menus = FALSE

	var
		moved = FALSE
		mouse_screen_loc

	MouseMove()
		// This enables MouseDrag to be called within the same atom.

	// Moves the mob to the mouse whenever the mouse is dragged.
	MouseDrag(src_object, over_object, src_location, over_location, src_control, over_control, params)
		var/list/p = params2list(params)

		/* This is kept up-to-date and used after the sleep.
			The effect is that only the last screen-loc of the current tick is used for movement.
			This is so we're not moving multiple times per tick, which keeps pixel gliding smooth.
		*/
		mouse_screen_loc = p["screen-loc"]

		if(moved) return
		moved = TRUE
		sleep world.tick_lag
		moved = FALSE

		/* With my Mouse Position and Pixel Movement libraries, the rest of this becomes:

			if(p["right"])
				mob.SetCenterTo(mouse_position.WorldPosition())
			else
				mob.PixelSlideTo(mouse_position.WorldPosition())

			But this is a good chance to show how awful life is without those libraries.
		*/
		var/regex/r = regex(@#(\d+):(\d+),(\d+):(\d+)#)
		if(!r.Find(mouse_screen_loc)) return
		var
			mouse_world_x = bound_x + text2num(r.group[2]) + (text2num(r.group[1]) - 1) * 32
			mouse_world_y = bound_y + text2num(r.group[4]) + (text2num(r.group[3]) - 1) * 32
			to_center_x = mob.bound_width / 2
			to_center_y = mob.bound_height / 2
			old_x = mob.step_x + (mob.x - 1) * 32 + mob.bound_x + to_center_x
			old_y = mob.step_y + (mob.y - 1) * 32 + mob.bound_y + to_center_y
			new_x = mouse_world_x
			new_y = mouse_world_y
			move_x = new_x - old_x
			move_y = new_y - old_y
		mob.step_size = 1 + max(abs(move_x), abs(move_y))
		if(p["right"])
			mob.step_x += move_x
			mob.step_y += move_y
		else
			mob.Move(mob.loc, 0, mob.step_x + move_x, mob.step_y)
			mob.Move(mob.loc, 0, mob.step_x, mob.step_y + move_y)
